      module crystaldata

!     ---------------------------------------------------------- description
!
!		SolidState          is SolidState on?
!		nMolInCell			numbers of moleculs in one cell
!		nCell				numbers of cells considered
!		nRepMet				numbers of repeated metals excluding the original
!		cellDim()			cell dimensions
!		RepMet(j,l)			metal j is repeated l times in the crystal
!		RotMatrix(l,3,3)	rotational matrix respect Orig to obtain the metal l (without translation)
!		Orig				tensore origine della cella
!		CyToOrig(3,3)		trasformation matrix of basis vector of Cyana frame to basis vector of Orig frame, in Cyana frame
!		CyRotMatrix(l,3,3)	rotational matrix respect Cyana to obtain the metal l (without translation)

      parameter (MAXDIPcry=10000,MAXTENScry=30,maxstrpsecry=50)
      logical SolidState, crypcsflag, crypsedia(maxdipcry), 
     *        crydip2flag(maxdipcry), crydipflag, OMflag
      integer nMolInCell, nCell, nRepMet, crystalmode, crytenso,ndipcry
      double precision cellDim(3), OMweight, OMtol, OMviol(3,3) 
      parameter (nMolInCell=4,nCell=27,nRepMet=nMolInCell*nCell-1)
      double precision :: CyToOrig(3,3), RotMatrix(nRepMet,3,3),
     *        CyRotMatrix(nRepMet,3,3),CyRotInv(nRepMet,3,3),OMRot(3,3)
      character symmetry*9
      integer cryIRESDIP(MAXDIPcry), cryNPROT(MAXDIPcry), 
     *        crynute(maxdipcry), cryNDTENSO, cryNTENSO, cryncrystal,
     *        cryINDXDIP(maxdipcry),
     *        cryIMET(maxdipcry),cryIAX(maxdipcry),cryIAY(maxdipcry),
     *        cryIAZ(maxdipcry),cryIPX(maxdipcry),cryIPY(maxdipcry),
     *        cryIPZ(maxdipcry),cryIPCN(maxdipcry)
      character cryNAMRESDIP*5(maxdipcry), cryNAMATDIP*5(maxdipcry)
      double precision cryPSHIFTOR(maxdipcry), cryTOLPROT(maxdipcry), 
     *     cryWPROT(maxdipcry), crypseperc, cryWDIP, crytoldip,
     *     cryPVIOL(maxdipcry),cryA1DIP(maxdipcry),cryA2DIP(maxdipcry)
      double precision crypshiftpse(maxdipcry)

      TYPE TENSOR
      double precision :: C(3),AX(3),AY(3),AZ(3) ! center and axes
      END TYPE TENSOR

      TYPE(TENSOR) :: RepMet(11,0:nRepMet)
      TYPE(TENSOR) :: Orig





      interface operator (.X.)
      module procedure v_prod
      end interface

      contains
  		function v_prod(a,b) result (p)
    	double precision,dimension(3),intent(in)  :: a,b
    	double precision,dimension(3)             :: p
    	p(1)=a(2)*b(3)-a(3)*b(2)
    	p(2)=a(3)*b(1)-a(1)*b(3)
    	p(3)=a(1)*b(2)-a(2)*b(1)
    	end function v_prod

      end

      SUBROUTINE initRotMatrix
      use cyanadata
      use crystaldata
C
      implicit double precision (a-h,o-z)
c
c
! matrix initialization. It needs to be "synchronized" with metal replications inside pseviol.f
      select case (symmetry)
      case('P21212','P212121','P2221','P222')
        l=1
        do ijk=1,nMolInCell,1
			   do j1=-1,1,1
			     do j2=-1,1,1
			      do j3=-1,1,1
                   if(j1.eq.0 .and. j2.eq.0 .and. j3.eq.0 ! to avoid replication of original metal
     *                                          .and. ijk.eq.1) then
			         ! do nothing
			       else
			        if(ijk.eq.1) then
			      	  RotMatrix(l,:,:)=0.0
			      	  do k = 1,3; RotMatrix(l,k,k) = 1.0; end do
			        endif
			        if(ijk.eq.2) then
					  RotMatrix(l,1,:)=(/-1.,0.,0./)
					  RotMatrix(l,2,:)=(/0.,-1.,0./)
					  RotMatrix(l,3,:)=(/0.,0.,1./)
			        endif
			        if(ijk.eq.3) then
			          RotMatrix(l,1,:)=(/-1.,0.,0./)
					  RotMatrix(l,2,:)=(/0.,1.,0./)
					  RotMatrix(l,3,:)=(/0.,0.,-1./)
			        endif
			        if(ijk.eq.4) then
			          RotMatrix(l,1,:)=(/1.,0.,0./)
					  RotMatrix(l,2,:)=(/0.,-1.,0./)
					  RotMatrix(l,3,:)=(/0.,0.,-1./)
			        endif
			        l=l+1

			       endif ! do nothing
			      enddo
			     enddo
			    enddo
	enddo
      case default
        CALL ERRMSG('Crystal symmetry not specified or unknown!')
        stop
      end select
      return
      end
