C ---------------------------------------------------------------------
C      FINDZEROCOORDS: finds the coordinates of the
C                      pseudoatoms AX, AY and AZ given the coordinates
C                      of tensor center and the ones of PX, PY, PZ and
C                      PCN (the center of the P-reference system).
C
C                      It reads direction cosines from a file called
C                      filenam.cos where filnam is the same as the 
C                      name given to the coordinate set.
C
C                      After calling this subroutine it is necessary
C                      to call GENER to obtain the actual coordinates
C                      CX, CY and CZ.
C                      N.B. All the A- and the P- pseudoatoms have a 
C                      lenght of 1.0 A from the corresponding center.
C
C                      Mauro A. Cremonini 07.02.96
C                   
C                      Now modified to use multiple tensors. See just the 
C                      addiction of the index running from 1 to NTENSO 
C                      (the number of different tensors).
C                  
C                      Mauro A. Cremonini 06/06/96
C
C                      DYANA version. Mauro A. Cremonini 06.12.96
C                     
C ---------------------------------------------------------------------
      SUBROUTINE FINDZEROCOORDS (iunit,mode)
      use cyanadata
      use crystaldata
C
      implicit double precision (a-h,o-z)
c
c

      INCLUDE 'orj.incl'
      INCLUDE 'rdc.incl'
      INCLUDE 'pse.incl'


! first cry tensors

      DO I=1,cryNTENSO*cryNDTENSO
        IPCN1=cryIPCN(I)
        IMET1=cryIMET(I)
        IPX1= cryIPX(I)
        IPY1= cryIPY(I)
        IPZ1= cryIPZ(I)
        IAX1= cryIAX(I)
        IAY1= cryIAY(I)
        IAZ1= cryIAZ(I)

        PCNX0=coo0(1,IPCN1)
        PCNY0=coo0(2,IPCN1)
        PCNZ0=coo0(3,IPCN1)

        CMEX0=coo0(1,IMET1)
        CMEY0=coo0(2,IMET1)
        CMEZ0=coo0(3,IMET1)


        PXX0=coo0(1,IPX1)-PCNX0
        PXY0=coo0(2,IPX1)-PCNY0
        PXZ0=coo0(3,IPX1)-PCNZ0
        PYX0=coo0(1,IPY1)-PCNX0
        PYY0=coo0(2,IPY1)-PCNY0
        PYZ0=coo0(3,IPY1)-PCNZ0
        PZX0=coo0(1,IPZ1)-PCNX0
        PZY0=coo0(2,IPZ1)-PCNY0
        PZZ0=coo0(3,IPZ1)-PCNZ0


        read (iunit,'(3e15.5)') txx,txy,txz
        read (iunit,'(3e15.5)') tyx,tyy,tyz
        read (iunit,'(3e15.5)') tzx,tzy,tzz
        read (iunit,'(2e15.5)') crya1dip(i),crya2dip(i)
        if ((crya1dip(i).ne.0).and.(crya2dip(i).ne.0)) then
        if (((abs(crya1dip(i)).le.1.0e-34).or.
     *                          (abs(crya1dip(i)).ge.1.0e-29))
     *   .or.(abs(crya2dip(i)).le.1.0e-34).or.
     *                          (abs(crya2dip(i)).ge.1.0e-29))
     *  then
        call wrnmsg('Magnetic susceptibility tensor '
     *            //'could be not in the right unit of measure (m^3)')
	goto 750
        endif
        end if
        ! the suscepibility magnetic tensor is converted in Angstrom and ppm
        ! divided by 12pi and the rhombic is multiplied by 1.5
        crya1dip(i)=crya1dip(i)/3.77e-35
        crya2dip(i)=1.5*crya2dip(i)/3.77e-35
750     continue   
        call putlin(4,'crya1dip for pse (=Dchi_ax): '
     *        //realp(crya1dip,8))
        call putlin(4,'crya2dip for pse (=3/2*Dchi_or): '
     *        //realp(crya2dip,8))
        coo0(1,IAX1) = PXX0*TXX+PYX0*TXY+PZX0*TXZ+CMEX0
        coo0(2,IAX1) = PXY0*TXX+PYY0*TXY+PZY0*TXZ+CMEY0
        coo0(3,IAX1) = PXZ0*TXX+PYZ0*TXY+PZZ0*TXZ+CMEZ0
        coo0(1,IAY1) = PXX0*TYX+PYX0*TYY+PZX0*TYZ+CMEX0
        coo0(2,IAY1) = PXY0*TYX+PYY0*TYY+PZY0*TYZ+CMEY0
        coo0(3,IAY1) = PXZ0*TYX+PYZ0*TYY+PZZ0*TYZ+CMEZ0
        coo0(1,IAZ1) = PXX0*TZX+PYX0*TZY+PZX0*TZZ+CMEX0
        coo0(2,IAZ1) = PXY0*TZX+PYY0*TZY+PZY0*TZZ+CMEY0
        coo0(3,IAZ1) = PXZ0*TZX+PYZ0*TZY+PZZ0*TZZ+CMEZ0
      ENDDO


! then pcs tensors

      DO I=1,NTENSO*NDTENSO
        IPCN1=IPCN(I)
        IMET1=IMET(I)
        IPX1= IPX(I)
        IPY1= IPY(I)
        IPZ1= IPZ(I)
        IAX1= IAX(I)
        IAY1= IAY(I)
        IAZ1= IAZ(I)

        PCNX0=coo0(1,IPCN1)
        PCNY0=coo0(2,IPCN1)
        PCNZ0=coo0(3,IPCN1)

        CMEX0=coo0(1,IMET1)
        CMEY0=coo0(2,IMET1)
        CMEZ0=coo0(3,IMET1)


        PXX0=coo0(1,IPX1)-PCNX0
        PXY0=coo0(2,IPX1)-PCNY0
        PXZ0=coo0(3,IPX1)-PCNZ0
        PYX0=coo0(1,IPY1)-PCNX0
        PYY0=coo0(2,IPY1)-PCNY0
        PYZ0=coo0(3,IPY1)-PCNZ0
        PZX0=coo0(1,IPZ1)-PCNX0
        PZY0=coo0(2,IPZ1)-PCNY0
        PZZ0=coo0(3,IPZ1)-PCNZ0


        read (iunit,'(3e15.5)') txx,txy,txz
        read (iunit,'(3e15.5)') tyx,tyy,tyz
        read (iunit,'(3e15.5)') tzx,tzy,tzz
        read (iunit,'(2e15.5)') a1dip(i),a2dip(i)
        if ((a1dip(i).ne.0).and.(a2dip(i).ne.0)) then
        if (((abs(a1dip(i)).le.1.0e-34).or.(abs(a1dip(i)).ge.1.0e-29))
     *   .or.(abs(a2dip(i)).le.1.0e-34).or.(abs(a2dip(i)).ge.1.0e-29))
     *  then
        call wrnmsg('Magnetic susceptibility tensor '
     *            //'could be not in the right unit of measure (m^3)')
	goto 700
        endif
        end if
        ! the suscepibility magnetic tensor is converted in Angstrom and ppm
        ! divided by 12pi and the rhombic is multiplied by 1.5
        a1dip(i)=a1dip(i)/3.77e-35
        a2dip(i)=1.5*a2dip(i)/3.77e-35
700     continue   
        call putlin(4,'a1dip for pse (=Dchi_ax): '//realp(a1dip,8))
        call putlin(4,'a2dip for pse (=3/2*Dchi_or): '//realp(a2dip,8))
        coo0(1,IAX1) = PXX0*TXX+PYX0*TXY+PZX0*TXZ+CMEX0
        coo0(2,IAX1) = PXY0*TXX+PYY0*TXY+PZY0*TXZ+CMEY0
        coo0(3,IAX1) = PXZ0*TXX+PYZ0*TXY+PZZ0*TXZ+CMEZ0
        coo0(1,IAY1) = PXX0*TYX+PYX0*TYY+PZX0*TYZ+CMEX0
        coo0(2,IAY1) = PXY0*TYX+PYY0*TYY+PZY0*TYZ+CMEY0
        coo0(3,IAY1) = PXZ0*TYX+PYZ0*TYY+PZZ0*TYZ+CMEZ0
        coo0(1,IAZ1) = PXX0*TZX+PYX0*TZY+PZX0*TZZ+CMEX0
        coo0(2,IAZ1) = PXY0*TZX+PYY0*TZY+PZY0*TZZ+CMEY0
        coo0(3,IAZ1) = PXZ0*TZX+PYZ0*TZY+PZZ0*TZZ+CMEZ0
      ENDDO


C     Always calculates inertia with coo0().
c
c   Now go for orientation tensor(s)
c

      DO J=1,NDTENSORJ
        I=J+MAXTENS/2
        IPCN1=IPCN(I)
        IMET1=IMET(I)
        IPX1= IPX(I)
        IPY1= IPY(I)
        IPZ1= IPZ(I)
        IAX1= IAX(I)
        IAY1= IAY(I)
        IAZ1= IAZ(I)

        PCNX0=coo0(1,IPCN1)
        PCNY0=coo0(2,IPCN1)
        PCNZ0=coo0(3,IPCN1)

        CMEX0=coo0(1,IMET1)
        CMEY0=coo0(2,IMET1)
        CMEZ0=coo0(3,IMET1)


        PXX0=coo0(1,IPX1)-PCNX0
        PXY0=coo0(2,IPX1)-PCNY0
        PXZ0=coo0(3,IPX1)-PCNZ0
        PYX0=coo0(1,IPY1)-PCNX0
        PYY0=coo0(2,IPY1)-PCNY0
        PYZ0=coo0(3,IPY1)-PCNZ0
        PZX0=coo0(1,IPZ1)-PCNX0
        PZY0=coo0(2,IPZ1)-PCNY0
        PZZ0=coo0(3,IPZ1)-PCNZ0


        read (iunit,'(3e15.5)') txx,txy,txz
        read (iunit,'(3e15.5)') tyx,tyy,tyz
        read (iunit,'(3e15.5)') tzx,tzy,tzz
        read (iunit,'(2e15.5)') a1dip(i),a2dip(i)
        ! the suscepibility magnetic tensor is converted in Angstrom and ppm
        ! divided by 12pi and the rhombic is multiplied by 1.5
        a1dip(i)=a1dip(i)/3.77e-35
        a2dip(i)=1.5*a2dip(i)/3.77e-35
        call putlin(4,'a1dip for orj (=Dchi_ax): '//realp(a1dip,8))
        call putlin(4,'a2dip for orj (=3/2*Dchi_or): '//realp(a2dip,8))
        
        coo0(1,IAX1) = PXX0*TXX+PYX0*TXY+PZX0*TXZ+CMEX0
        coo0(2,IAX1) = PXY0*TXX+PYY0*TXY+PZY0*TXZ+CMEY0
        coo0(3,IAX1) = PXZ0*TXX+PYZ0*TXY+PZZ0*TXZ+CMEZ0
        coo0(1,IAY1) = PXX0*TYX+PYX0*TYY+PZX0*TYZ+CMEX0
        coo0(2,IAY1) = PXY0*TYX+PYY0*TYY+PZY0*TYZ+CMEY0
        coo0(3,IAY1) = PXZ0*TYX+PYZ0*TYY+PZZ0*TYZ+CMEZ0
        coo0(1,IAZ1) = PXX0*TZX+PYX0*TZY+PZX0*TZZ+CMEX0
        coo0(2,IAZ1) = PXY0*TZX+PYY0*TZY+PZY0*TZZ+CMEY0
        coo0(3,IAZ1) = PXZ0*TZX+PYZ0*TZY+PZZ0*TZZ+CMEZ0
      ENDDO

c
c   Now go for rdc
c

      DO J=1,NDTENSRDC
        I=J+MAXTENS/2
        IPCN1=IPCN(I)
        IMET1=IMET(I)
        IPX1= IPX(I)
        IPY1= IPY(I)
        IPZ1= IPZ(I)
        IAX1= IAX(I)
        IAY1= IAY(I)
        IAZ1= IAZ(I)

        PCNX0=coo0(1,IPCN1)
        PCNY0=coo0(2,IPCN1)
        PCNZ0=coo0(3,IPCN1)

        CMEX0=coo0(1,IMET1)
        CMEY0=coo0(2,IMET1)
        CMEZ0=coo0(3,IMET1)


        PXX0=coo0(1,IPX1)-PCNX0
        PXY0=coo0(2,IPX1)-PCNY0
        PXZ0=coo0(3,IPX1)-PCNZ0
        PYX0=coo0(1,IPY1)-PCNX0
        PYY0=coo0(2,IPY1)-PCNY0
        PYZ0=coo0(3,IPY1)-PCNZ0
        PZX0=coo0(1,IPZ1)-PCNX0
        PZY0=coo0(2,IPZ1)-PCNY0
        PZZ0=coo0(3,IPZ1)-PCNZ0


        read (iunit,'(3e15.5)') txx,txy,txz
        read (iunit,'(3e15.5)') tyx,tyy,tyz
        read (iunit,'(3e15.5)') tzx,tzy,tzz
        read (iunit,'(2e15.5)') a1dip(i),a2dip(i)
        a1dip(i)=a1dip(i)/3.77e-35
        a2dip(i)=1.5*a2dip(i)/3.77e-35
        call putlin(4,'a1dip for rdc (=Dchi_ax): '//realp(a1dip,8))
        call putlin(4,'a2dip for rdc (=3/2*Dchi_or): '//realp(a2dip,8))
        
        coo0(1,IAX1) = PXX0*TXX+PYX0*TXY+PZX0*TXZ+CMEX0
        coo0(2,IAX1) = PXY0*TXX+PYY0*TXY+PZY0*TXZ+CMEY0
        coo0(3,IAX1) = PXZ0*TXX+PYZ0*TXY+PZZ0*TXZ+CMEZ0
        coo0(1,IAY1) = PXX0*TYX+PYX0*TYY+PZX0*TYZ+CMEX0
        coo0(2,IAY1) = PXY0*TYX+PYY0*TYY+PZY0*TYZ+CMEY0
        coo0(3,IAY1) = PXZ0*TYX+PYZ0*TYY+PZZ0*TYZ+CMEZ0
        coo0(1,IAZ1) = PXX0*TZX+PYX0*TZY+PZX0*TZZ+CMEX0
        coo0(2,IAZ1) = PXY0*TZX+PYY0*TZY+PZY0*TZZ+CMEY0
        coo0(3,IAZ1) = PXZ0*TZX+PYZ0*TZY+PZZ0*TZZ+CMEZ0
      ENDDO


      call gener
      call inerta
      RETURN
      END


      SUBROUTINE FINDOMROT (iunit,mode)
      use cyanadata
      use crystaldata
C
      implicit double precision (a-h,o-z)
c
c

      INCLUDE 'orj.incl'
      INCLUDE 'rdc.incl'
      INCLUDE 'pse.incl'

! Origin-Metal matrix alignment in Origin coordinates
        read (iunit,'(3e15.5)') OMRot(1,1),OMRot(1,2),OMRot(1,3)
        read (iunit,'(3e15.5)') OMRot(2,1),OMRot(2,2),OMRot(2,3)
        read (iunit,'(3e15.5)') OMRot(3,1),OMRot(3,2),OMRot(3,3)

      END
