C    ------------------------------------------------------------------
C    GETcryDIP:     reads a file containing the pseudocontact shift
C                constraints for crystal. 
C                Format is:
C
C                I3,1X,2A5,1X,F7.2,1X,I1,A1,f5.2,1x,f5.2,1x,I2
C
C                that corresponds to:
C
C   cryIRESDIP()    residue number;
C   cryNAMRESDIP()  residue name;
C   cryNAMATDIP()   atom name;
C   cryPSHIFTOR()   pseudocontact shift (original);
C   QQDIA        if letter 'd' the averaged experimental shift is compared
C                to the averaged experimental shift (in this case weight 
C                is doubled);
C   cryNPROT()      number of atoms whose calculated shifts must be averaged;
C   cryTOLPROT()    tolerance on calculated shift;
C   cryWPROT()      weight of the individual contraints (multiplies wdip).
C
C         
C                Mauro A. Cremonini 24/01/95
C                Mauro A. Cremonini 07/02/96
C
C
C   cryNUTE()      index counting the different tensors
C
C
C     A new modification has been done: the pseudocontact input now contains
C     N entry more than the number of shifts to be used. These correspond to the
C     PCN center of the residue bearing the Nth tensor. It is assumed that the residue 
C     has PX, PY and PZ attached to PCN. It is also assumed that a ME 
C     atom attached to AX, AY and AZ exist. If they all do not, the program stops  
C     complaining about the residue.
C               
C                 Mauro Rinaldelli modification from Mauro Cremonini 


C     ------------------------------------------------------------------

      SUBROUTINE GETCRYDIP(IUNIT,mode)
      use cyanadata
      use crystaldata
C
      implicit double precision (a-h,o-z)
c
C
      CHARACTER LINE*80,qqdia*1
 
      NDIPcry=0
  10  CONTINUE
        READ(iunit,'(A)',END=99) LINE
        NDIPcry=NDIPcry+1
        if (ndipcry.gt.maxdipcry) then
          CALL ERRMSG('Too many pseudocontact shifts!')
          ndipcry=0
          return
        endif
        READ(LINE,'(I3,1X,2A5,1X,F7.2,1X,I1,a1,f5.2,1x,f5.2,1x,I2)') 
     *  cryIRESDIP(NDIPcry),cryNAMRESDIP(NDIPcry),cryNAMATDIP(NDIPcry),
     *       cryPSHIFTOR(NDIPcry),cryNPROT(NDIPcry),qqdia,
     *       cryTOLPROT(NDIPcry),cryWPROT(NDIPcry),cryNUTE(NDIPcry)
        if (qqdia.eq.'d') then  
          crypsedia(ndipcry)=.true.
        else
          crypsedia(ndipcry)=.false.
        endif
        IF (cryNUTE(NDIPcry).LT.1) cryNUTE(NDIPcry)=1
        IF (cryNUTE(NDIPcry).GT.cryNDTENSO) STOP
        IF (cryNPROT(NDIPcry).EQ.0) cryNPROT(NDIPcry)=1
        IF (cryTOLPROT(NDIPcry).EQ.0.0) cryTOLPROT(NDIPcry)=cryTOLDIP
        IF (cryWPROT(NDIPcry).EQ.0.0) cryWPROT(NDIPcry)=1.
        if (crywprot(ndipcry).lt.0.) crywprot(ndipcry)=0.
        crydip2flag(ndipcry)=.true.
      GOTO 10
 
  99  i=1
      do while (i.le.ndipcry)
        if (crypsedia(i)) then
          if (crynprot(i).eq.2.and.crywprot(i).gt.0.) then
            crypshiftpse(i)=(crypshiftor(i)+crypshiftor(i+1))/2
            crypshiftpse(i+1)=crypshiftpse(i)
            crywprot(i+1)=4.
C wprot(i+1)=2.
            i=i+1
          else
            CALL ERRMSG('Only geminal couples!')
            ndipcry=0
            return
          endif
        else 
          crypshiftpse(i)=crypshiftor(i)
        endif
        i=i+1
      enddo
       
C     Modifica prof. Luchinat
      do i=1,ndipcry
       crytolprot(i)=max(crytolprot(i),abs(crypshiftpse(i)*crypseperc))
      enddo


      mustbezero=8
      DO 20 I=1,NDIPcry
        DO 30 K=1,NR
          IF (cryIRESDIP(I).EQ.IR(K).AND.
     *                   I.LE.NDIPcry-cryNTENSO*cryNDTENSO) THEN
            DO 40 J=IFIRA(K), IFIRA(K+1)-1
              IF (cryNAMATDIP(I).EQ.ANAM(J)) THEN
                cryINDXDIP(I)=J
                GO TO 20
              END IF
  40        CONTINUE
            CALL ERRMSG('GETDIP:illegal atom name '//cryNAMATDIP(I))
	    ndipcry=0
	    return
          ELSE IF (cryIRESDIP(I).EQ.IR(K).AND.
     *             I.GT.cryNDIP-cryNTENSO*cryNDTENSO) THEN
            itensprog=i-ndipcry+cryntenso*cryNDTENSO
            DO 41 J=IFIRA(K), IFIRA(K+1)-1
              IF (ANAM(J).EQ.'ME   ') THEN
                cryIMET(itensprog)=J
                mustbezero=mustbezero-1
              elseif (ANAM(J).EQ.'PX   ') then
                cryipx(itensprog)=j
                mustbezero=mustbezero-1
              elseif (ANAM(J).EQ.'PY   ') then
                cryipy(itensprog)=j
                mustbezero=mustbezero-1
              elseif (ANAM(J).EQ.'PZ   ') then
                cryipz(itensprog)=j
                mustbezero=mustbezero-1
              elseif (ANAM(J).EQ.'AX   ') then
                cryiax(itensprog)=j
                mustbezero=mustbezero-1
              elseif (ANAM(J).EQ.'AY   ') then
                cryiay(itensprog)=j
                mustbezero=mustbezero-1
              elseif (ANAM(J).EQ.'AZ   ') then
                cryiaz(itensprog)=j
                mustbezero=mustbezero-1
              elseif (ANAM(J).EQ.'PCN  ') then
                cryipcn(itensprog)=j
                mustbezero=mustbezero-1
              endif
  41        CONTINUE
            if (mustbezero.eq.0) then
              mustbezero=8
              goto 20
            endif
          CALL ERRMSG('GETDIP:illegal tensor stuff! '//crynamresdip(i))
	    ndipcry=0
	    return
          ELSE IF (cryIRESDIP(I).NE.IR(K).AND.K.EQ.NR) THEN
C            print *, 'Reside  number = ',iresdip(i)
          CALL ERRMSG('GETDIP:illegal residue number '//cryNAMATDIP(I))
	    ndipcry=0
	    return
          END IF
  30    CONTINUE
  20  CONTINUE
      NDIPcry=NDIPcry-cryntenso*cryNDTENSO


      cryDIPFLAG=.TRUE.
	write(6,*)'crydipflag=',crydipflag  !ATTENZIONE

      open (88,file='getdipcry.out')
      do i=1,ndipcry
       if (crypsedia(i)) then
         qqdia='d'
       else
         qqdia=' '
       endif
       WRITE(88,'(I3,1X,2A5,1X,F7.2,1X,I1,a1,f5.2,1x,f5.2,1x,
     *            f7.2,2x,i5,1x,i5,1x,I2)')
     *              cryIRESDIP(i),cryNAMRESDIP(i),cryNAMATDIP(i),
     *              cryPSHIFTPSE(i),cryNPROT(i),qqdia,
     *              cryTOLPROT(i),cryWPROT(i),cryPSHIFTOR(i),
     *              cryindxdip(i),iap(cryindxdip(i)),cryNUTE(i)
      enddo
      close(88)

      RETURN
      END
