! Copyright (c) 2002 Peter Guntert. All rights reserved.
!  -----------------------------------------------------------------
!
!         PSEVIOL: calculates and stores in the array PVIOL() the
!                  violation of the pseudocontact shift constraints.
!                  It also calculates the pseudocontact target
!                  function (F).
!                  M. A. Cremonini 06.12.96
!
!  ------------------------------------------------------------------

      function PSEVIOL()

      use cyanadata
      use crystaldata ! parametri per lo stato solido

      implicit double precision (a-h,o-z)
      double precision temp(3,3),Rij(3),Ril(3),fdip,fdipcry
      double precision OMRotMy(3,3),OrMx1(3,3),MeMx1(3,3)
!     *  ,calcsing0(5000)!,GG1,GG2! ! Rij è la posizione del nucleo non metallico rispetto al metallo considerato
c
      INCLUDE 'pse.incl'

      ! R_MET(indice metallo) sono i tensori metallici rispetto a cyana. Servono solo se siamo
      ! in stato solido, quindi con l'ultimo tensore che è l'origine degli assi
      ! (ecco perché i vettori sono fino a ntenso-1)
      ! RO_MET(indice metallo) coordinate del centro rispetto all'origine della cella
      TYPE(TENSOR)	::	R_MET(cryntenso-1), RO_MET(cryntenso-1)

      ZNUL=0.

      fdip=0.0
      fdipcry=0.0
      fdipOM=0.0
      OMtol=0.1 ! in questo momento l'assegnazione di OMtol è fatta solo qui

!-----------------------------------------------------------------------
!		if Solidstate (solid state modification)
!-----------------------------------------------------------------------
       if (crydipflag) then
       
        do i=1,ndipcry
         calcsing(i)=0.
         calctot(i)=0.
        enddo

         select case (symmetry)
         case ('P21212')
      ! coordinates of tensor frame O with respect to Cyana frame
	  Orig%C(1)=coo(1,cryIMET(cryntenso))
	  Orig%C(2)=coo(2,cryIMET(cryntenso))
	  Orig%C(3)=coo(3,cryIMET(cryntenso))

	  ! unit vectors coordinates
	   Orig%AX(:)=coo(:,cryIAX(cryntenso))-Orig%C(:)
	   Orig%AY(:)=coo(:,cryIAY(cryntenso))-Orig%C(:)
	   Orig%AZ(:)=coo(:,cryIAZ(cryntenso))-Orig%C(:)

       ! matrix that transforms Cyana basis vector in Origin basis vectors
       ! coordinates transform with the inverse matrix (in this case it coincides with the transpose)
       ! Thus this is also the matrix that transform coordinates from Origin frame to Cyana frame
       ! e.g. (1,0,0) of Origin become the vector Orig%AX in Cyana
	   CyToOrig(:,1)=Orig%AX
	   CyToOrig(:,2)=Orig%AY
	   CyToOrig(:,3)=Orig%AZ

	  ! CyToOrig=M
	  ! CyRotMatrix=M*R*M^Transp
	  ! CyRotInv=M*R^Transp*M^Transp=CyRotMatrix^Transp

	  do l=1,nRepMet
	  	temp(:,:)=TRANSPOSE(CyToOrig)
	  	temp=MATMUL(RotMatrix(l,:,:),temp)
	  	CyRotMatrix(l,:,:)=MATMUL(CyToOrig,temp)
	  	CyRotInv(l,:,:)=TRANSPOSE(CyRotMatrix(l,:,:)) ! inverse=transpose
	  end do



	  ! metals replication
	  DO j=1,cryntenso-1 
            ! acquisition of metal coordinates with respect to Cyana Frame. R_MET(metalIndex)
			R_MET(j)%C(:)=coo(:,cryIMET(j))
			R_MET(j)%AX(:)=coo(:,cryIAX(j))-R_MET(j)%C(:)
			R_MET(j)%AY(:)=coo(:,cryIAY(j))-R_MET(j)%C(:)
			R_MET(j)%AZ(:)=coo(:,cryIAZ(j))-R_MET(j)%C(:)

			! (R_MET(j)%C-Orig%C) are metal coordinates with respect to Origin, in Cyana Frame. RO_MET(metalIndex)
			! I trasform them in Origin frame
			RO_MET(j)%C=MATMUL(TRANSPOSE(CyToOrig),(R_MET(j)%C-Orig%C))

		    ! metal replication inside the cell
			! P21212 Symmetry. Replication of metal inside the cell (4 molecules) and of 27 cells
	        l=1
	        do ijk=1,nMolInCell,1
			    do j1=-1,1,1
			     do j2=-1,1,1
			      do j3=-1,1,1
					if(j1.eq.0 .and. j2.eq.0 .and. j3.eq.0 .and. ijk.eq.1) then  ! to avoid the replication of the original one
			         ! do nothing
			        else
			         select case (ijk)
			         case(1)
                     ! replicated metal coordinates with respect to Origin
			         ! Attention: here are replicated in this way, but in general I have to apply a rotation matrix to RO_MET
			         ! (at the end i need to add Origin coordinates)
			RepMet(j,l)%C(1)=RO_MET(j)%C(1)+dfloat(j1)*cellDim(1)
			RepMet(j,l)%C(2)=RO_MET(j)%C(2)+dfloat(j2)*cellDim(2)
			RepMet(j,l)%C(3)=RO_MET(j)%C(3)+dfloat(j3)*cellDim(3)
			         case(2)
			RepMet(j,l)%C(1)=-RO_MET(j)%C(1)+dfloat(j1)*cellDim(1)
			RepMet(j,l)%C(2)=-RO_MET(j)%C(2)+dfloat(j2)*cellDim(2)
			RepMet(j,l)%C(3)=+RO_MET(j)%C(3)+dfloat(j3)*cellDim(3)
			         case(3)
					 RepMet(j,l)%C(1)=-RO_MET(j)%C(1)+
     *								dfloat(j1)*cellDim(1)+cellDim(1)/2.
			         RepMet(j,l)%C(2)=+RO_MET(j)%C(2)+
     *         						dfloat(j2)*cellDim(2)+cellDim(2)/2.
			         RepMet(j,l)%C(3)=-RO_MET(j)%C(3)+
     *         						dfloat(j3)*cellDim(3)
			         case(4)
			         RepMet(j,l)%C(1)=+RO_MET(j)%C(1)+
     *		         				dfloat(j1)*cellDim(1)+cellDim(1)/2.
			         RepMet(j,l)%C(2)=-RO_MET(j)%C(2)+
     *		         				dfloat(j2)*cellDim(2)+cellDim(2)/2.
			         RepMet(j,l)%C(3)=-RO_MET(j)%C(3)+
     *		         				dfloat(j3)*cellDim(3)
			         end select
					 ! axis X rotation, in Cyana frame
				RepMet(j,l)%AX=MATMUL(CyRotMatrix(l,:,:),R_MET(j)%AX) ! è corretto il prodotto matrice vettore con MATMUL?
					 ! Y axis
				RepMet(j,l)%AY=MATMUL(CyRotMatrix(l,:,:),R_MET(j)%AY)
					 ! Z axis
				RepMet(j,l)%AZ=MATMUL(CyRotMatrix(l,:,:),R_MET(j)%AZ)

					 ! I want coordinates with respect to Cyana frame
				RepMet(j,l)%C=MATMUL(CyToOrig(:,:),RepMet(j,l)%C)+Orig%C

					 l=l+1
					endif ! do nothing
	              end do
	             end do
	            end do
	        end do
			! in RepMet(j,l) I have coordinates of metals l, coming from j, in Cyana frame

			! I complete RepMet(j,l) adding the original in l=0:
			RepMet(j,0)=R_MET(j)
	  ENDDO
	  ! end of metal replication

         case ('P212121')
      ! 
	  Orig%C(1)=coo(1,cryIMET(cryntenso))
	  Orig%C(2)=coo(2,cryIMET(cryntenso))
	  Orig%C(3)=coo(3,cryIMET(cryntenso))

	  ! 
	   Orig%AX(:)=coo(:,cryIAX(cryntenso))-Orig%C(:)
	   Orig%AY(:)=coo(:,cryIAY(cryntenso))-Orig%C(:)
	   Orig%AZ(:)=coo(:,cryIAZ(cryntenso))-Orig%C(:)

      !
	   CyToOrig(:,1)=Orig%AX
	   CyToOrig(:,2)=Orig%AY
	   CyToOrig(:,3)=Orig%AZ

      !
	  do l=1,nRepMet
	  	temp(:,:)=TRANSPOSE(CyToOrig)
	  	temp=MATMUL(RotMatrix(l,:,:),temp)
	  	CyRotMatrix(l,:,:)=MATMUL(CyToOrig,temp)
	  	CyRotInv(l,:,:)=TRANSPOSE(CyRotMatrix(l,:,:)) 
	  end do



	  ! metal replication
	  DO j=1,cryntenso-1 ! capire se inserire anche ndtenso
			! 
			R_MET(j)%C(:)=coo(:,cryIMET(j))
			R_MET(j)%AX(:)=coo(:,cryIAX(j))-R_MET(j)%C(:)
			R_MET(j)%AY(:)=coo(:,cryIAY(j))-R_MET(j)%C(:)
			R_MET(j)%AZ(:)=coo(:,cryIAZ(j))-R_MET(j)%C(:)

			! 
			RO_MET(j)%C=MATMUL(TRANSPOSE(CyToOrig),(R_MET(j)%C-Orig%C))

		    ! metal replication inside the cell
			! P212121 symmetry
	        l=1
	        do ijk=1,nMolInCell,1
			    do j1=-1,1,1
			     do j2=-1,1,1
			      do j3=-1,1,1
					if(j1.eq.0 .and. j2.eq.0 .and. j3.eq.0 .and. ijk.eq.1) then 
			         ! do nothing
			        else
			         select case (ijk)
			         case(1) 
			RepMet(j,l)%C(1)=RO_MET(j)%C(1)+dfloat(j1)*cellDim(1)
			RepMet(j,l)%C(2)=RO_MET(j)%C(2)+dfloat(j2)*cellDim(2)
			RepMet(j,l)%C(3)=RO_MET(j)%C(3)+dfloat(j3)*cellDim(3)
			         case(2)
			RepMet(j,l)%C(1)=-RO_MET(j)%C(1)+dfloat(j1)*cellDim(1)+cellDim(1)/2.
			RepMet(j,l)%C(2)=-RO_MET(j)%C(2)+dfloat(j2)*cellDim(2)
			RepMet(j,l)%C(3)=+RO_MET(j)%C(3)+dfloat(j3)*cellDim(3)+cellDim(3)/2.
			         case(3)
					 RepMet(j,l)%C(1)=-RO_MET(j)%C(1)+
     *								dfloat(j1)*cellDim(1)
			         RepMet(j,l)%C(2)=+RO_MET(j)%C(2)+
     *         						dfloat(j2)*cellDim(2)+cellDim(2)/2.
			         RepMet(j,l)%C(3)=-RO_MET(j)%C(3)+
     *         						dfloat(j3)*cellDim(3)+cellDim(3)/2.
			         case(4)
			         RepMet(j,l)%C(1)=+RO_MET(j)%C(1)+
     *		         				dfloat(j1)*cellDim(1)+cellDim(1)/2.
			         RepMet(j,l)%C(2)=-RO_MET(j)%C(2)+
     *		         				dfloat(j2)*cellDim(2)+cellDim(2)/2.
			         RepMet(j,l)%C(3)=-RO_MET(j)%C(3)+
     *		         				dfloat(j3)*cellDim(3)
			         end select
				RepMet(j,l)%AX=MATMUL(CyRotMatrix(l,:,:),R_MET(j)%AX) ! è corretto il prodotto matrice vettore con MATMUL?
				RepMet(j,l)%AY=MATMUL(CyRotMatrix(l,:,:),R_MET(j)%AY)
				RepMet(j,l)%AZ=MATMUL(CyRotMatrix(l,:,:),R_MET(j)%AZ)

				RepMet(j,l)%C=MATMUL(CyToOrig(:,:),RepMet(j,l)%C)+Orig%C

					 l=l+1
					endif ! do nothing
	              end do
	             end do
	            end do
	        end do

			RepMet(j,0)=R_MET(j)

	  ENDDO
	  ! end of metal replication

          case ('P2221')
      ! 
	  Orig%C(1)=coo(1,cryIMET(cryntenso))
	  Orig%C(2)=coo(2,cryIMET(cryntenso))
	  Orig%C(3)=coo(3,cryIMET(cryntenso))

	  ! 
	   Orig%AX(:)=coo(:,cryIAX(cryntenso))-Orig%C(:)
	   Orig%AY(:)=coo(:,cryIAY(cryntenso))-Orig%C(:)
	   Orig%AZ(:)=coo(:,cryIAZ(cryntenso))-Orig%C(:)

      !
	   CyToOrig(:,1)=Orig%AX
	   CyToOrig(:,2)=Orig%AY
	   CyToOrig(:,3)=Orig%AZ

	  ! 
	  ! CyToOrig=M
	  ! CyRotMatrix=M*R*M^Transp dove M=CyToOrig
	  ! CyRotInv=M*R^Transp*M^Transp=CyRotMatrix^Transp

	  do l=1,nRepMet
	  	temp(:,:)=TRANSPOSE(CyToOrig)
	  	temp=MATMUL(RotMatrix(l,:,:),temp)
	  	CyRotMatrix(l,:,:)=MATMUL(CyToOrig,temp)
	  	CyRotInv(l,:,:)=TRANSPOSE(CyRotMatrix(l,:,:)) 
	  end do



	  ! metal replication
	  DO j=1,cryntenso-1 
			! 
			R_MET(j)%C(:)=coo(:,cryIMET(j))
			R_MET(j)%AX(:)=coo(:,cryIAX(j))-R_MET(j)%C(:)
			R_MET(j)%AY(:)=coo(:,cryIAY(j))-R_MET(j)%C(:)
			R_MET(j)%AZ(:)=coo(:,cryIAZ(j))-R_MET(j)%C(:)

			! 
			RO_MET(j)%C=MATMUL(TRANSPOSE(CyToOrig),(R_MET(j)%C-Orig%C))

		    ! 
			! P2221 symmetry
	        l=1
	        do ijk=1,nMolInCell,1
			    do j1=-1,1,1
			     do j2=-1,1,1
			      do j3=-1,1,1
					if(j1.eq.0 .and. j2.eq.0 .and. j3.eq.0 .and. ijk.eq.1) then  ! per evitare la replicazione della molecola originale
			         ! do nothing
			        else
			         select case (ijk)
			         case(1) 
			         ! 
			RepMet(j,l)%C(1)=RO_MET(j)%C(1)+dfloat(j1)*cellDim(1)
			RepMet(j,l)%C(2)=RO_MET(j)%C(2)+dfloat(j2)*cellDim(2)
			RepMet(j,l)%C(3)=RO_MET(j)%C(3)+dfloat(j3)*cellDim(3)
			         case(2)
			RepMet(j,l)%C(1)=-RO_MET(j)%C(1)+dfloat(j1)*cellDim(1)
			RepMet(j,l)%C(2)=-RO_MET(j)%C(2)+dfloat(j2)*cellDim(2)
			RepMet(j,l)%C(3)=+RO_MET(j)%C(3)+dfloat(j3)*cellDim(3)+cellDim(3)/2.
			         case(3)
					 RepMet(j,l)%C(1)=-RO_MET(j)%C(1)+
     *								dfloat(j1)*cellDim(1)
			         RepMet(j,l)%C(2)=+RO_MET(j)%C(2)+
     *         						dfloat(j2)*cellDim(2)
			         RepMet(j,l)%C(3)=-RO_MET(j)%C(3)+
     *         						dfloat(j3)*cellDim(3)+cellDim(3)/2.
			         case(4)
			         RepMet(j,l)%C(1)=+RO_MET(j)%C(1)+
     *		         				dfloat(j1)*cellDim(1)
			         RepMet(j,l)%C(2)=-RO_MET(j)%C(2)+
     *		         				dfloat(j2)*cellDim(2)
			         RepMet(j,l)%C(3)=-RO_MET(j)%C(3)+
     *		         				dfloat(j3)*cellDim(3)
			         end select
				RepMet(j,l)%AX=MATMUL(CyRotMatrix(l,:,:),R_MET(j)%AX) 
				RepMet(j,l)%AY=MATMUL(CyRotMatrix(l,:,:),R_MET(j)%AY)
				RepMet(j,l)%AZ=MATMUL(CyRotMatrix(l,:,:),R_MET(j)%AZ)

				RepMet(j,l)%C=MATMUL(CyToOrig(:,:),RepMet(j,l)%C)+Orig%C
					 l=l+1
					endif ! do nothing
	              end do
	             end do
	            end do
	        end do

			RepMet(j,0)=R_MET(j)
	  ENDDO
	  ! end of metal replication

          case ('P222')
      ! 
	  Orig%C(1)=coo(1,cryIMET(cryntenso))
	  Orig%C(2)=coo(2,cryIMET(cryntenso))
	  Orig%C(3)=coo(3,cryIMET(cryntenso))

	  ! 
	   Orig%AX(:)=coo(:,cryIAX(cryntenso))-Orig%C(:)
	   Orig%AY(:)=coo(:,cryIAY(cryntenso))-Orig%C(:)
	   Orig%AZ(:)=coo(:,cryIAZ(cryntenso))-Orig%C(:)

	   ! 
	   CyToOrig(:,1)=Orig%AX
	   CyToOrig(:,2)=Orig%AY
	   CyToOrig(:,3)=Orig%AZ

	  ! 
	  ! CyToOrig=M
	  ! CyRotMatrix=M*R*M^Transp dove M=CyToOrig
	  ! CyRotInv=M*R^Transp*M^Transp=CyRotMatrix^Transp

	  do l=1,nRepMet
	  	temp(:,:)=TRANSPOSE(CyToOrig)
	  	temp=MATMUL(RotMatrix(l,:,:),temp)
	  	CyRotMatrix(l,:,:)=MATMUL(CyToOrig,temp)
	  	CyRotInv(l,:,:)=TRANSPOSE(CyRotMatrix(l,:,:)) ! 
	  end do



	  ! metals replication
	  DO j=1,cryntenso-1 
			! 
			R_MET(j)%C(:)=coo(:,cryIMET(j))
			R_MET(j)%AX(:)=coo(:,cryIAX(j))-R_MET(j)%C(:)
			R_MET(j)%AY(:)=coo(:,cryIAY(j))-R_MET(j)%C(:)
			R_MET(j)%AZ(:)=coo(:,cryIAZ(j))-R_MET(j)%C(:)

			! 
			RO_MET(j)%C=MATMUL(TRANSPOSE(CyToOrig),(R_MET(j)%C-Orig%C))

		    ! 
			! P222 symmetry
	        l=1
	        do ijk=1,nMolInCell,1
			    do j1=-1,1,1
			     do j2=-1,1,1
			      do j3=-1,1,1
					if(j1.eq.0 .and. j2.eq.0 .and. j3.eq.0 .and. ijk.eq.1) then  
			         ! do nothing
			        else
			         select case (ijk)
			         case(1) 
			RepMet(j,l)%C(1)=RO_MET(j)%C(1)+dfloat(j1)*cellDim(1)
			RepMet(j,l)%C(2)=RO_MET(j)%C(2)+dfloat(j2)*cellDim(2)
			RepMet(j,l)%C(3)=RO_MET(j)%C(3)+dfloat(j3)*cellDim(3)
			         case(2)
			RepMet(j,l)%C(1)=-RO_MET(j)%C(1)+dfloat(j1)*cellDim(1)
			RepMet(j,l)%C(2)=-RO_MET(j)%C(2)+dfloat(j2)*cellDim(2)
			RepMet(j,l)%C(3)=+RO_MET(j)%C(3)+dfloat(j3)*cellDim(3)
			         case(3)
					 RepMet(j,l)%C(1)=-RO_MET(j)%C(1)+
     *								dfloat(j1)*cellDim(1)
			         RepMet(j,l)%C(2)=+RO_MET(j)%C(2)+
     *         						dfloat(j2)*cellDim(2)
			         RepMet(j,l)%C(3)=-RO_MET(j)%C(3)+
     *         						dfloat(j3)*cellDim(3)
			         case(4)
			         RepMet(j,l)%C(1)=+RO_MET(j)%C(1)+
     *		         				dfloat(j1)*cellDim(1)
			         RepMet(j,l)%C(2)=-RO_MET(j)%C(2)+
     *		         				dfloat(j2)*cellDim(2)
			         RepMet(j,l)%C(3)=-RO_MET(j)%C(3)+
     *		         				dfloat(j3)*cellDim(3)
			         end select
				RepMet(j,l)%AX=MATMUL(CyRotMatrix(l,:,:),R_MET(j)%AX) 
				RepMet(j,l)%AY=MATMUL(CyRotMatrix(l,:,:),R_MET(j)%AY)
				RepMet(j,l)%AZ=MATMUL(CyRotMatrix(l,:,:),R_MET(j)%AZ)

					 ! 
				RepMet(j,l)%C=MATMUL(CyToOrig(:,:),RepMet(j,l)%C)+Orig%C

					 l=l+1
					endif ! do nothing
	              end do
	             end do
	            end do
	        end do

			RepMet(j,0)=R_MET(j)
	  ENDDO
	  ! end metal replication

         case default
          CALL ERRMSG('Crystal symmetry not specified or unknown!')
         end select ! symmetry selection

	  ! calculation of metals influence on every nucelus      
      DO j=1,cryntenso-1
       DO l=0,nRepMet ! starting from 0 I consider also original metal

      	do i=1,ndipcry
          IF((cryNUTE(I).EQ.j)) THEN    
             i_ker=cryINDXDIP(i) 
             Ril(:)=coo(:,i_ker)-RepMet(j,l)%C(:)
             R2=Ril(1)**2+Ril(2)**2+Ril(3)**2
             R=SQRT(R2)
!            if(R.gt.100) cycle
             R5=R**5
             GG1=(3.0*(DOT_PRODUCT(Ril,RepMet(j,l)%AZ))**2-R2)/R5
             GG2=(DOT_PRODUCT(Ril,RepMet(j,l)%AX)**2-
     *         (DOT_PRODUCT(Ril,RepMet(j,l)%AY))**2)/R5
             calcsing(i)=calcsing(i)+
     *                (cryA1DIP(j)*GG1+cryA2DIP(j)*GG2) !calcsing(i) is the pcs of nucelus i
          ENDIF
        enddo
       END DO ! cycle over l until nRepMet
      END DO

        i=1
	    do while (i.le.ndipcry)
	      if (crydip2flag(i)) then  
	          nprot1=crynprot(i) 
	          costmult=1./float(nprot1)
	          calc=calcsing(i)
	          do h=1,nprot1-1
	            i=i+1
	            calc=calc+calcsing(i)
	          enddo
	          calctot(i)=calc*costmult !calctot sembra essere il pcs del singolo nucleo i, mediato se occorre.

	          cdiff=calctot(i)-crypshiftpse(i) !pcs calcolata - pcs sperimentale del nucleo i

	          crypviol(i)=sign(max(abs(cdiff)-crytolprot(i),znul),cdiff)
	          fdipcry=fdipcry+crywprot(i)*crypviol(i)**2 
	      endif
	      i=i+1
	    enddo

        if (OMflag) then
          
          OrMx1(:,1) = Orig%AX
          OrMx1(:,2) = Orig%AY
          OrMx1(:,3) = Orig%AZ

          MeMx1(:,1) = RepMet(1,0)%AX
          MeMx1(:,2) = RepMet(1,0)%AY
          MeMx1(:,3) = RepMet(1,0)%AZ

          OMRotMY = MATMUL(transpose(MeMx1),OrMx1)

          do ii=1,3
            do jj=1,3
              cdiff = OMRotMY(ii,jj) - OMRot(ii,jj)
              OMviol(ii,jj) = sign(max(abs(cdiff)-OMtol,znul),cdiff)
              fdipOM = fdipOM + OMviol(ii,jj)**2
            end do
          end do
        endif

      end if ! if crydipflag

!---------------------------------------------------------------------------------------
!      else ! if not SolidState (fine modifica stato solido)

      if (dipflag) then
!---------------------------------------------------------------------------------------

        do i=1,ndip
         calcsing(i)=0.
         calctot(i)=0.
        enddo

      do iq=1,ntenso*NDTENSO

    !
    !  ANtonio
    !
        CMEX=coo(1,IMET(iq))
        CMEY=coo(2,IMET(iq))
        CMEZ=coo(3,IMET(iq))

        AXX=coo(1,IAX(iq))-CMEX
        AXY=coo(2,IAX(iq))-CMEY
        AXZ=coo(3,IAX(iq))-CMEZ
        AYX=coo(1,IAY(iq))-CMEX
        AYY=coo(2,IAY(iq))-CMEY
        AYZ=coo(3,IAY(iq))-CMEZ
        AZX=coo(1,IAZ(iq))-CMEX
        AZY=coo(2,IAZ(iq))-CMEY
        AZZ=coo(3,IAZ(iq))-CMEZ

        do i=1,ndip

          IF((ntenso.ne.1).or.(NUTE(I).EQ.IQ)) THEN
c          IF((NUTE(I).EQ.IQ)) THEN 
             J=INDXDIP(I)
          
             HX=coo(1,J)-CMEX
             HY=coo(2,J)-CMEY
             HZ=coo(3,J)-CMEZ
             R2=HX**2+HY**2+HZ**2
             R=SQRT(R2)
             R5=R**5
             GG1=(3.0*(HX*AZX+HY*AZY+HZ*AZZ)**2-R2)/R5
             GG2=((HX*AXX+HY*AXY+HZ*AXZ)**2-
     *         (HX*AYX+HY*AYY+HZ*AYZ)**2)/R5
             calcsing(i)=calcsing(i)+
     *                (A1DIP(iq)*GG1+A2DIP(iq)*GG2)

          ENDIF
        enddo
      enddo

      i=1
      do while (i.le.ndip)
        if (dip2flag(i)) then 
          nprot1=nprot(i) 
          costmult=1./float(nprot1)
          calc=calcsing(i)
          do j=1,nprot1-1
            i=i+1
            calc=calc+calcsing(i)
          enddo
          calctot(i)=calc*costmult !calctot sembra essere il pcs del singolo nucleo i, mediato se occorre.

c
c   modifica 27/6/2000
c
c	if(pshift(i).eq.0)then
c          cdiff=(calctot(i)-pshift(i))
c	else
c          cdiff=(calctot(i)-pshift(i))/pshift(i)/10.
c	endif
cc
c     fine modifica

          cdiff=calctot(i)-pshiftpse(i) !pcs calcolata - pcs sperimentale del nucleo i

          pviol(i)=sign(max(abs(cdiff)-tolprot(i),znul),cdiff)
!         The position of the violation in PVIOL() corresponds to
!         the last atom whose chemical shift is averaged.
          fdip=fdip+wprot(i)*pviol(i)**2 !violazione totale pesata
        endif
        i=i+1
      enddo

      end if ! if dipflag
!-----------------------------------------------------------------------
!      endif		! end if SolidState
!       end select
!-----------------------------------------------------------------------

      pseviol=wdip*fdip+crywdip*fdipcry+OMweight*fdipOM

      RETURN
      END


